/*
 * Copyright: Nicholas P Wiebe (2008-2009) and Irmtraud M Meyer (2008-2009)
 * License: licensed under the GNU General Public License version 3 (GPLv3)
 * EvolModel.h
 * Singleton object that stores the models sequence evolution in paired and unpaired RNA
 * (Same model as in SimulFold, PFold)
 */

#ifndef EVOLMODEL_H_
#define EVOLMODEL_H_


class EvolModel {
public:
	virtual ~EvolModel();

	typedef double array_of_4_double_t[4];
	typedef double array_of_16_double_t[16];


	static double* ePiUnpaired[4];
	static array_of_4_double_t* eVUnpaired[4];
	static double* eDUnpaired[4];
	static array_of_4_double_t* eWUnpaired[4];


	static double* ePiPaired[4][4];
	static array_of_16_double_t* eVPaired[4][4];
	static double* eDPaired[4][4];
	static array_of_16_double_t* eWPaired[4][4];

	static double r_p;
	static double r_ns[4];
	static double ePiSingle0[4];
	static double eVSingle0[4][4];
	static double eDSingle0[4];
	static double eWSingle0[4][4];

	static double ePiDouble00[16];
	static double eVDouble00[16][16];
	static double eDDouble00[16];
	static double eWDouble00[16][16];

	static double ePiDouble11[16];		// Same for all other Double## matrices
	static double eVDouble11[16][16];
	static double eDDouble11[16];
	static double eWDouble11[16][16];

	static double eVDouble12[16][16];
	static double eDDouble12[16];
	static double eWDouble12[16][16];

	static double eVDouble13[16][16];
	static double eDDouble13[16];
	static double eWDouble13[16][16];

	static double eVDouble21[16][16];
	static double eDDouble21[16];
	static double eWDouble21[16][16];

	static double eVDouble22[16][16];
	static double eDDouble22[16];
	static double eWDouble22[16][16];

	static double eVDouble23[16][16];
	static double eDDouble23[16];
	static double eWDouble23[16][16];

	static double eVDouble31[16][16];
	static double eDDouble31[16];
	static double eWDouble31[16][16];

	static double eVDouble32[16][16];
	static double eDDouble32[16];
	static double eWDouble32[16][16];

	static double eVDouble33[16][16];
	static double eDDouble33[16];
	static double eWDouble33[16][16];

	static double ePiSingle1[4];
	static double eVSingle1[4][4];
	static double eDSingle1[4];
	static double eWSingle1[4][4];

	static double ePiSingle2[4];
	static double eVSingle2[4][4];
	static double eDSingle2[4];
	static double eWSingle2[4][4];

	static double ePiSingle3[4];
	static double eVSingle3[4][4];
	static double eDSingle3[4];
	static double eWSingle3[4][4];

private:
	EvolModel(); //singleton
};

#endif /* EVOLMODEL_H_ */
