/*
 * LikelihoodCache.cpp
 *
 *  Created on: Mar 7, 2012
 *      Author: jeffrp
 */

#include "LikelihoodCache.h"

#include <cassert>
#include <iostream>
using namespace std;

LikelihoodCache::LikelihoodCache(int seqLength) {

	this->length = seqLength;

	/*
	// triangular matrix implementation
	 * ... saves space, but we actually need values outside of the triangle for the shuffled helices.
	 * Need full square matrix instead..
	this->index = new int [seqLength];
	for (int j = 0; j < seqLength; j++) {
		this->index[j] = j * (j + 1) / 2 - j;
	}

	this->pairedCache = new double[this->index[seqLength-1] + seqLength-1];
	for (int j = 1; j < seqLength; j++) {
		for (int i = 0; i < j; i++) {
			this->pairedCache[this->index[j] + i] = 1.0;
		}
	}
	*/

	this->pairedCache = new double*[seqLength];
	for (int i = 0; i < seqLength; i++) {
		this->pairedCache[i] = new double[seqLength];
		for (int j = 0; j < seqLength; j++) {
			this->pairedCache[i][j] = 1.0;
		}
	}


	// one-dimensional unpaired cache
	this->unpairedCache = new double[seqLength];
	for (int i = 0; i < seqLength; i++) {
		unpairedCache[i] = 1.0;
	}
}

LikelihoodCache::~LikelihoodCache() {
	/*
	 * From triangular matrix implementation
	 *delete this->index;
	this->index = 0;
	*/
	for (int i = 0; i < this->length; i++) {
		delete[] this->pairedCache[i];
		this->pairedCache[i] = 0;
	}

	delete[] this->pairedCache;
	this->pairedCache = 0;

	delete[] this->unpairedCache;
	this->unpairedCache = 0;
}




bool LikelihoodCache::isCached(int i, int j) {
	/*
	int idx;
	if (i < j) {
		idx = this->index[j] + i;
	} else {
		idx = this->index[i] + j;
	}
	return (this->pairedCache[idx] < 0);
	*/
	return(this->pairedCache[i][j] < 0);
}
double LikelihoodCache::getCached(int i, int j) {
	/*
	int idx;
	if (i < j) {
		idx = this->index[j] + i;
	} else {
		idx = this->index[i] + j;
	}
	return this->pairedCache[idx];
	*/
	return this->pairedCache[i][j];
}
void LikelihoodCache::setCached(int i, int j, double likelihood) {
	/*
	int idx;
	if (i < j) {
		idx = this->index[j] + i;
	} else {
		idx = this->index[i] + j;
	}
	*/
	this->pairedCache[i][j] = likelihood;
}



bool LikelihoodCache::isCached(int i) {
	assert(i < this->length);
	return(unpairedCache[i] < 0);
}
double LikelihoodCache::getCached(int i) {
	assert(i < this->length);
	return(unpairedCache[i]);
}
void LikelihoodCache::setCached(int i, double likelihood) {
	assert(i < this->length);
	unpairedCache[i] = likelihood;
}
